#!/bin/zsh
# Calculate speedup ratios for pairs of error estimates.
# This is done in inefficient shell code for convenience.
# This scripts expects the folder "timing" to be filled
# with data files following a specific naming convention.
emulate -L zsh

problems=(B1-e3 B1-e4 1B2-e3 1B2-e4 6B2-e3 6B2-e4 12B2-e3 12B2-e4 RCD-e0)
typeset -A timings ratios
kcols=({3..10})

function print_ratio() {
  [[ $# != 2 ]] && return 1
  case $1 in
    ble-bac) printf "BAC/LE / BAC/ST = %.2f\n" $2 ;;
    sci-bac) printf "SCI/ST / BAC/ST = %.2f\n" $2 ;;
    loi-bac) printf "LOI/LE / BAC/ST = %.2f\n" $2 ;;
    loi-sci) printf "LOI/LE / SCI/ST = %.2f\n" $2 ;;
  esac
}

# Calculate the time averages from the data.
for prob in $problems; do
  e=${prob#*-e}
  for k in $kcols; do
    for tol in {4,6,8}; do
      for est in loi sci bac ble; do
        if [[ $est == ble ]]; then
          K=$((k-1))
          EST=bac
        else
          K=$k
          EST=$est
        fi
        key="${prob}-${est}-k${k}t${tol}"
        f="timing/${prob%-*}-k${K}t${tol}e${e}-${EST}.out"
        # average from run iterations
        if grep -vq '[a-zA-DF-Z]' $f; then
          t=( $(cat $f) )
          if (( ${#t} )); then
            avg=$(( ( ${(j:+:)t} ) / ${#t} ))
            timings[${key}]=$avg
          fi
        fi
      done
      for ratio in ble-bac sci-bac loi-bac loi-sci; do
        x=${prob}; y=k${k}t${tol}; a=${ratio%-*}; b=${ratio#*-}
        [[ -z $timings[$x-$a-$y] ]] && continue
        [[ -z $timings[$x-$b-$y] ]] && continue
        ratios[$x-$ratio-$y]=$(( $timings[$x-$a-$y] / $timings[$x-$b-$y] ))
      done
    done
  done
done

# compute average-of-ratios per problem
for prob in $problems; do
  print
  print "$prob average-of-ratios:"
  for ratio in ble-bac sci-bac loi-bac loi-sci; do
    t=()
    for key in ${(k)ratios}; do
      [[ $key =~ ^${prob}-${ratio} ]] && t+=$ratios[$key]
    done
    (( ${#t} )) || continue
    avg=$(( ( ${(j:+:)t} ) / ${#t} ))
    print_ratio $ratio $avg
    #print '             '${#t}
  done
done

print

# compute average-of-ratios per kcol
for k in $kcols; do
  print
  print "kcol=${k} average-of-ratios:"
  for ratio in ble-bac sci-bac loi-bac loi-sci; do
    t=()
    for key in ${(k)ratios}; do
      [[ $key =~ ${ratio}-k${k} ]] && t+=$ratios[$key]
    done
    (( ${#t} )) || continue
    avg=$(( ( ${(j:+:)t} ) / ${#t} ))
    print_ratio $ratio $avg
    #print '             '${#t}
  done
done

print

# compute overall average-of-ratios
print
print "Overall average-of-ratios:"
for ratio in ble-bac sci-bac loi-bac loi-sci; do
  t=()
  for key in ${(k)ratios}; do
    [[ $key =~ ${ratio} ]] && t+=$ratios[$key]
  done
  (( ${#t} )) || continue
  avg=$(( ( ${(j:+:)t} ) / ${#t} ))
  print_ratio $ratio $avg
  #print '             '${#t}
done

print

# compute average per kcol and tol to find fastest kcol
for tol in {4,6,8}; do
  for k in $kcols; do
    print
    print "tol=${tol},kcol=${k} average:"
    for est in bac ble sci loi; do
      t=()
      for key in ${(k)timings}; do
        [[ $key =~ ${est}-k${k}t${tol} ]] && t+=$timings[$key]
      done
      (( ${#t} )) || continue
      avg=$(( ( ${(j:+:)t} ) / ${#t} ))
      printf "%s = %.2f\n" ${est:u} ${avg}
    done
  done
done
