#!/usr/bin/env python3
# Make work precision diagrams for the various kcol choices
# of a single scheme.

import matplotlib as mpl
mpl.use('PS')
import matplotlib.pyplot as plt
import numpy as np
import sys, re, itertools
from collections import OrderedDict

try:
  burg, eps = (int(s) for s in re.match(r'B(\d)e(\d)', sys.argv[1]).groups())
except:
  print('usage: %s <problem code> [<skip rows>]' % sys.argv[0])
  sys.exit(1)

# OrderedDict is prefered over vanilla dictionaries for iterating over.
# A list would be okay too, but dictionary syntax is convenient.
datafiles = OrderedDict([
  pair for sublist in [
    [
      ('BAC/ST, $kcol={:d}$'.format(kcol), 'work-precision/B{0:d}-k{2:d}e{1:d}-bac.out'.format(burg, eps, kcol)),
      ('BAC/LE, $kcol={:d}$'.format(kcol), 'work-precision/B{0:d}-k{2:d}e{1:d}-ble.out'.format(burg, eps, kcol)),
      ('SCI/ST, $kcol={:d}$'.format(kcol), 'work-precision/B{0:d}-k{2:d}e{1:d}-sci.out'.format(burg, eps, kcol)),
      ('LOI/LE, $kcol={:d}$'.format(kcol), 'work-precision/B{0:d}-k{2:d}e{1:d}-loi.out'.format(burg, eps, kcol))
    ] for kcol in range(2,12)
  ] for pair in sublist
])

data = OrderedDict()
for key, fname in datafiles.items():
  try:
    data[key] = np.loadtxt(fname, usecols=(0,1,2), unpack=True)
  except:
    print('could not retrieve \'%s\' data from \'%s\'.' % (key, fname))

mpl.rcParams['figure.figsize'] = (24,18)
mpl.rcParams['legend.labelspacing'] = 0.02
for n, scheme in enumerate(['BAC/ST', 'BAC/LE', 'SCI/ST', 'LOI/LE']):

  # make subplots on a 2x2 grid
  ax = plt.subplot(2,2,n+1)
  cc = itertools.cycle(list('rgcbmky')+['#C2085B','#9F55ED','#07A812','#C27E08'])

  # collect table data for this subplot in these lists
  rlabels = []
  efail = []
  tbad = []
  tfail = []

  for i, (key, (error, time, tols)) in enumerate(data.items()):

    if not key.startswith(scheme): continue

    # count failures in the data (from errors and tolerances)
    label = key.split()[-1]
    tot = 91
    nfail = tot - len(error)
    rlabels.append(label)
    efail.append(nfail)
    tbad.append(np.sum(error - tols > 0))
    tfail.append(np.sum(error - tols*10 > 0))

    # fit data to log(y) = m*log(x) + b
    A = np.vstack([np.log(error), np.ones_like(error)]).T
    m, b = np.linalg.lstsq(A, np.log(time))[0]
    trend = np.exp(m*np.log(error)+b)

    c = next(cc)  # next colour in the cycle

    # plot data points and trendline
    ax.loglog(error, time, c=c, ls='', marker='.', ms=4)
    ax.loglog(error, trend, c=c, ls='-', lw=1, label=label)


  # figure-level formatting (see matplotlibrc file for default settings)
  ax.set_xlabel('$L^2$-norm error')
  ax.set_ylabel('CPU Time (s)')
  ax.set_xlim(ax.get_xlim()[::-1]) # reverse x axis
  ax.set_title(scheme)
  ax.legend(loc=2,title=r'data fit to $\log{y} = m \log{x} + b$') # upper left
  ax.grid(True, which='major', axis='both', ls=':')
  ax.grid(False, which='minor', axis='both')

  # tabulate other data
  cells = [tup for tup in zip(efail, tbad, tfail)]
  table = ax.table(cellText=cells, cellLoc='center', rowLabels=rlabels,
      colLabels=['# failures','# err > tol','# err > 10*tol'],
      colWidths=[0.13]*3, loc='lower right')

plt.gcf().suptitle('%s Burgers\' equation, $\epsilon = 10^{-%d}$'
    % ('$1^{st}$' if burg == 1 else '$2^{nd}$', eps),
    fontsize=18)
plt.savefig('%s-wp-kcols.eps' % sys.argv[1])
