#!/usr/bin/env python3
# Make work/precision diagrams comparing schemes
# against a BAC/ST baseline.

import matplotlib as mpl
mpl.use('PS')
import matplotlib.pyplot as plt
import numpy as np
import sys, re
from collections import OrderedDict

try:
  burg, eps, kcol = (int(s) for s in re.match(r'B(\d)e(\d)k(\d+)', sys.argv[1]).groups())
except:
  print('usage: %s <problem code> [<skip rows>]' % sys.argv[0])
  sys.exit(1)

# OrderedDict is prefered over vanilla dictionaries for iterating over.
# A list would be okay too, but dictionary syntax is convenient.
datafiles = OrderedDict([
  ('BAC/ST', 'work-precision/B{0:d}-k{2:d}e{1:d}-bac.out'.format(burg, eps, kcol)),
  ('BAC/LE', 'work-precision/B{0:d}-k{2:d}e{1:d}-ble.out'.format(burg, eps, kcol)),
  ('SCI/ST', 'work-precision/B{0:d}-k{2:d}e{1:d}-sci.out'.format(burg, eps, kcol)),
  ('LOI/LE', 'work-precision/B{0:d}-k{2:d}e{1:d}-loi.out'.format(burg, eps, kcol))
])

data = OrderedDict()
for key, fname in datafiles.items():
  try:
    data[key] = np.loadtxt(fname, usecols=(0,1), unpack=True)
  except:
    print('could not retrieve %s data from %s.' % (key, fname))
    # The LOI scheme cannot be used when kcol = 3,
    # but the work-accuracy plots are still interesting to see
    # as they compare BACOL at its lowest kcol with SCI/ST.
    if not (kcol == 3 and key.startswith('LOI')): sys.exit(1)

# Create the baseline using a fit of the BAC/ST data.
# Note that base is a function.
error, time = data['BAC/ST']
A = np.vstack([np.log(error), np.ones_like(error)]).T
base_m, base_b = np.linalg.lstsq(A, np.log(time))[0]
base = lambda x: np.exp(base_m*np.log(x)+base_b)

ax = plt.axes() # simple case; no subplots
marks = '^osd'
lines = ['-','--','-','--']
colours = ['k','#9F55ED','#07A812','#C27E08']
mpl.rcParams['legend.labelspacing'] = 0.02
for i, (key, (error, time)) in enumerate(data.items()):

  # fit data to log(y) = m*log(x) + b
  A = np.vstack([np.log(error), np.ones_like(error)]).T
  m, b = np.linalg.lstsq(A, np.log(time))[0]
  trend = np.exp(m*np.log(error)+b)

  # compare data and trendline to baseline
  time_ratio = time / base(error)
  trend_ratio = trend / base(error)

  # make a linear fir of time_ratio
  A = np.vstack([np.log(error), np.ones_like(error)]).T
  m, b = np.linalg.lstsq(A, time_ratio)[0]
  time_ratio_trend = m*np.log(error)+b

  # call out failures in the legend
  tot = 91
  nfail = tot - len(error)
  label = key + (' (%d / %d failed)'%(nfail, tot) if nfail else '')

  # plot the trendline only
  # ax.semilogx(error, trend_ratio, c=colours[i], ls=lines[i], label=label, lw=2)
  ax.semilogx(error, time_ratio_trend, c=colours[i], ls=lines[i], label=label, lw=2)

  # the data points are distracting on this plot
  # ax.semilogx(error, time_ratio, c=colours[i], ls='', marker=marks[i], ms=4)

# figure-level formatting (see matplotlibrc file for default settings)
ax.set_xlabel('$L^2$-norm error')
ax.set_ylabel('Time relative to BAC/ST (fitted)')
ax.set_xlim(ax.get_xlim()[::-1])  # reverse x axis
ax.set_ylim((0, max(ax.get_ylim()[1],1.5)))  # keep y limits consistent between plots
ax.legend(loc=1, fontsize=10)  #, title=r'data fit to $\log{y} = m \log{x} + b$')
ax.grid(True, which='major', axis='both', ls=':')
ax.grid(False, which='minor', axis='both')

plt.savefig('%s-wp-rel.eps' % sys.argv[1])
