#!/usr/bin/env python3
# Make work/precision diagrams comparing four schemes
# Connect unsorted data points with line segments and point out failures.

import matplotlib as mpl
mpl.use('PS')
import matplotlib.pyplot as plt
import numpy as np
import sys, re
from collections import OrderedDict

try:
  burg, eps, kcol = (int(s) for s in re.match(r'B(\d)e(\d)k(\d+)', sys.argv[1]).groups())
except:
  print('usage: %s <problem code> [<skip rows>]' % sys.argv[0])
  sys.exit(1)

# OrderedDict is prefered over vanilla dictionaries for iterating over.
# A list would be okay too, but dictionary syntax is convenient.
datafiles = OrderedDict([
  ('BAC/ST', 'work-precision/B{0:d}-k{2:d}e{1:d}-bac.out'.format(burg, eps, kcol)),
  ('BAC/LE', 'work-precision/B{0:d}-k{2:d}e{1:d}-ble.out'.format(burg, eps, kcol)),
  ('SCI/ST', 'work-precision/B{0:d}-k{2:d}e{1:d}-sci.out'.format(burg, eps, kcol)),
  ('LOI/LE', 'work-precision/B{0:d}-k{2:d}e{1:d}-loi.out'.format(burg, eps, kcol))
])

data = OrderedDict()
for key, fname in datafiles.items():
  try:
    data[key] = np.loadtxt(fname, usecols=(0,1), unpack=True)
  except:
    print('could not retrieve %s data from %s.' % (key, fname))
    # The LOI scheme cannot be used when kcol = 3,
    # but the work-accuracy plots are still interesting to see
    # as they compare BACOL at its lowest kcol with SCI/ST.
    if not (kcol == 3 and key.startswith('LOI')): sys.exit(1)

ax = plt.axes() # simple case; no subplots
colours = ['#C2085B','#9F55ED','#07A812','#C27E08']
mpl.rcParams['legend.labelspacing'] = 0.02
for i, (key, (error, time)) in enumerate(data.items()):

  # call out failures in the legend
  tot = 91
  nfail = tot - len(error)
  label = key + (' (%d / %d failed)'%(nfail, tot) if nfail else '')

  # plot the data points connected by line segments
  # they are ordered by decreasing tolerance input, not by accuracy output
  ax.loglog(error, time, c=colours[i], lw=1, label=label)

# figure-level formatting (see matplotlibrc file for default settings)
ax.set_xlabel('$L^2$-norm error')
ax.set_ylabel('CPU Time (s)')
ax.set_xlim(ax.get_xlim()[::-1]) # reverse x axis
ax.legend(loc=2, fontsize=10) # upper left
ax.grid(True, which='major', axis='both', ls=':')
ax.grid(False, which='minor', axis='both')

plt.savefig('%s-wp-squiggle.eps' % sys.argv[1])
